Function Load-SharePoint-Powershell
{
	If ((Get-PsSnapin |?{$_.Name -eq "Microsoft.SharePoint.PowerShell"})-eq $null)
	{
    	Write-Host "- Loading SharePoint Powershell Snapin"
   		$PSSnapin = Add-PsSnapin Microsoft.SharePoint.PowerShell -ErrorAction SilentlyContinue | Out-Null
	}
}

Function GetAdminWebApplicationUrls
{
	$websvc = [Microsoft.SharePoint.Administration.SPWebService]::AdministrationService
	$adminurls = @()
	foreach ($webapp in $websvc.WebApplications) 
	{
		$adminurls = $adminurls + $webapp.GetResponseUri(([Microsoft.SharePoint.Administration.SPUrlZone]::Default)).ToString() 
	}
	return $adminurls
}

Function AddFarmSolution([string] $solutionName) 
{
	if (IsFarmSolutionAdded($solutionName)) 
	{
		Write-Host "- The solution $solutionName is already added"	
	} else
	{
		Add-SPSolution -LiteralPath $solutionName
	}
}

Function InstallFarmSolution([string] $solutionName) {
	foreach ($adminurl in GetAdminWebApplicationUrls)
	{
		Write-Host -NoNewLine "- Deploying $solutionName to $adminurl"
		Install-SPSolution Identity $solutionName WebApplication $adminurl -GACDeployment -Force:$true
		WaitForTimer($solutionName)
	}

	Write-Host -NoNewLine "- Deploying $solutionName to all content Web applications"
	Install-SPSolution Identity $solutionName AllWebApplications -GACDeployment -Force:$true
	WaitForTimer($solutionName)
}

Function UninstallFarmSolution([string] $solutionName) {
	Write-Host -NoNewLine "- Retracting $solutionName from all content Web applications"
	Uninstall-SPSolution Identity $solutionName AllWebApplications -Confirm:$false
	WaitForTimer($solutionName)

	foreach ($adminurl in GetAdminWebApplicationUrls)
	{
		Write-Host -NoNewLine "- Retracting $solutionName from $adminurl"
		Uninstall-SPSolution Identity $solutionName WebApplication $adminurl -Confirm:$false
		WaitForTimer($solutionName)
	}
}

Function RemoveFarmSolution([string] $solutionName) {
	if (IsFarmSolutionAdded($solutionName)) 
	{
		Write-Host "- Removing solution $solutionName"
		Remove-SPSolution -Identity $solutionName -Confirm:$false
	} else
	{
		Write-Host "- The solution $solutionName is already removed"	
	}
}

Function IsFarmSolutionAdded([string] $solutionName) {
    $solution = Get-SPSolution | Where { 
        ($solutionName -eq $_.Name) 
    } 
    if($solution -eq $null) { 
        return $false 
    } else { 
        return $true
    }
} 

Function WaitForTimer([string] $solutionName)
{
	do
	{
		Start-Sleep 2
		$retryCount++
		Write-Host "." -NoNewLine
		if($retryCount -gt 60)
		{
			Write-Host ""
			Write-Error "Timeout deploying solution $solutionName"
			break
		}
	} while ((Get-SPSolution $solutionName).JobExists)

	Write-Host ""
	$solution = (Get-SPSolution $solutionName)
	if($solution.LastOperationResult -gt 2)
	{
		Write-Error $solution.LastOperationDetails
	}
}

Function EnsureEventSource([string] $source) 
{
	if ([System.Diagnostics.EventLog]::SourceExists($source) -eq $false) {
		Write-Host "- Creating Windows Event Log source"
		[System.Diagnostics.EventLog]::CreateEventSource($source, "Application")
	}
}

Function Pause
{
	Write-Host "Press any key to continue ..."
	$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
}